#include "CPU_Ticks.h"
#include <time.h>

//+***********************************************************************+//
// Global Variables
//+***********************************************************************+//

//-------------------------------------------------------------------------//
// Global variable to hold the time stamp counter value when things are reset
//-------------------------------------------------------------------------//
__int64	StartTickCount	= 0;

//-------------------------------------------------------------------------//
// Global variable to hold the conversion between clock ticks and elapsed time
//-------------------------------------------------------------------------//
double	TicksToSec		= 1;

//+***********************************************************************+//
// Internal Dll Functions
//+***********************************************************************+//

//-------------------------------------------------------------------------//
// Function to calibrate the cpu clock in sec
//-------------------------------------------------------------------------//
void CalibrateTimeStampCounter(void)
{
	// Set the number of data points used to calibrate the clock
	const int	n	= 20;

	double	Sx	= 0;
	double	Sy	= 0;
	double	Sxx	= 0;
	double	Sxy	= 0;
	double	x;
	double	y;
	clock_t	t;
	clock_t	t1;

	// Initialize the counter to some well defined value
	InitTimeStampCounter();

	// Measure elapsed time and elapsed tick count a number of times and fit
	// a straight line to the data.
	t1	= clock() + 1;
	for(int i=0;i<n;++i){
		
		// Wait until the clock ticks over then quickly grab the tick count.
		// This tries to keep the latency for all the data points as close
		// to constant as possible.  For better accuracy, launch this as a
		// separate thread and monitor things for ~10 sec.
		while((t = clock()) < t1);
		x	= DblElapsedTicks();
		t1	= t+1;

		// Add the new data point to the sums
		y	= (double)t;
		Sx+=	x;
		Sy+=	y;
		Sxx+=	x*x;
		Sxy+=	x*y;
	}

	// Compute the slope of the line and convert the clock() time to sec.
	TicksToSec	= (n*Sxy - Sx*Sy)/((n*Sxx - Sx*Sx)*CLOCKS_PER_SEC);
}

//+***********************************************************************+//
// Mathcad functions
//+***********************************************************************+//

//-------------------------------------------------------------------------//
// Read the number of cpu ticks since things were last reset
//-------------------------------------------------------------------------//

// Function Info structure for Mathcad
FUNCTIONINFO ReadCpuTicks = 
{
	"ReadCpuTicks",
	"Dummy",
	"Reads the number of clock ticks on the CPU since the timer was reset.",
	(LPCFUNCTION)fReadCpuTicks,
	COMPLEX_SCALAR,
	1,
	{COMPLEX_SCALAR}
};

// Function definition
LRESULT  fReadCpuTicks(
        LPCOMPLEXSCALAR		CurrentTickCount,
		LPCCOMPLEXSCALAR	Dummy)
{
	// Return the time difference in units of clock ticks
	CurrentTickCount->real	= DblElapsedTicks();

	// normal return
	return 0;
}

//-------------------------------------------------------------------------//
// Reset the cpu tick count
//-------------------------------------------------------------------------//

// Function Info structure for Mathcad
FUNCTIONINFO ResetCpuTicks = 
{
	"ResetCpuTicks",
	"Dummy",
	"Resets the timer count to the current time stamp value",
	(LPCFUNCTION)fResetCpuTicks,
	COMPLEX_SCALAR,
	1,
	{COMPLEX_SCALAR}
};

// Function Definition
LRESULT  fResetCpuTicks(
        LPCOMPLEXSCALAR		Zero,
		LPCCOMPLEXSCALAR	Dummy)
{
	// Set the timer to the current time stamp counter reading.
	InitTimeStampCounter();

	// Return Zero=0 since a double cannot hold an __int64 value

	// normal return
	return 0;
}

//-------------------------------------------------------------------------//
// Read the time in seconds since things were last reset
//-------------------------------------------------------------------------//

// Function Info structure for Mathcad
FUNCTIONINFO ReadTime = 
{
	"ReadTime",
	"Dummy",
	"Compute the time in sec since the timer was reset.",
	(LPCFUNCTION)fReadTime,
	COMPLEX_SCALAR,
	1,
	{COMPLEX_SCALAR}
};

// Function definition
LRESULT  fReadTime(
        LPCOMPLEXSCALAR		TimeDiffInSec,
		LPCCOMPLEXSCALAR	Dummy)
{

	// Return the time difference in units of seconds
	TimeDiffInSec->real	= DblElapsedTicks()*TicksToSec;

	// normal return
	return 0;
}

//-------------------------------------------------------------------------//
// Get the Cpu clock speed in Mhz
//-------------------------------------------------------------------------//

// Function Info structure for Mathcad
FUNCTIONINFO ReadCpuSpeed = 
{
	"ReadCpuSpeed",
	"Dummy",
	"Get the Cpu clock speed in Mhz.",
	(LPCFUNCTION)fReadCpuSpeed,
	COMPLEX_SCALAR,
	1,
	{COMPLEX_SCALAR}
};

// Function definition
LRESULT  fReadCpuSpeed(
        LPCOMPLEXSCALAR		CpuMhz,
		LPCCOMPLEXSCALAR	Dummy)
{
	// Compute the speed in Mhz
	CpuMhz->real	= 1.0E-6/TicksToSec;

	// normal return
	return 0;
}

//-------------------------------------------------------------------------//
// Convert time in clock ticks to time in seconds
//-------------------------------------------------------------------------//

// Function Info structure for Mathcad
FUNCTIONINFO CpuTicksToSec = 
{
	"CpuTicksToSec",
	"TimeInTicks",
	"Convert the time in ticks to time in sec",
	(LPCFUNCTION)fCpuTicksToSec,
	COMPLEX_SCALAR,
	1,
	{COMPLEX_SCALAR}
};

// Function definition
LRESULT  fCpuTicksToSec(
        LPCOMPLEXSCALAR		TimeInSec,
		LPCCOMPLEXSCALAR	TimeInTicks)
{
	// Compute the speed in Mhz
	TimeInSec->real	= TimeInTicks->real*TicksToSec;

	// normal return
	return 0;
}

//-------------------------------------------------------------------------//
// Convert time in seconds to time in clock ticks
//-------------------------------------------------------------------------//

// Function Info structure for Mathcad
FUNCTIONINFO SecToCpuTicks = 
{
	"SecToCpuTicks",
	"TimeInSec",
	"Convert the time in sec to time in ticks",
	(LPCFUNCTION)fSecToCpuTicks,
	COMPLEX_SCALAR,
	1,
	{COMPLEX_SCALAR}
};

// Function definition
LRESULT  fSecToCpuTicks(
        LPCOMPLEXSCALAR		TimeInTicks,
		LPCCOMPLEXSCALAR	TimeInSec)
{
	// Compute the speed in Mhz
	TimeInTicks->real	= TimeInSec->real/TicksToSec;

	// normal return
	return 0;
}

